; Copyright (c) 1994 Adam Karpierz
; SPDX-License-Identifier: Zlib

EXPORTS
    crc_init                     @1
    crc_update                   @2
    crc_final                    @3
    crc_model                    @4
    crc_model_by_name            @5
    crc_predefined_model_by_name @6
    crc_predefined_models        @7
